/*
 * AMTP player - tool playing sound received via IEEE 1394, Audio
 *               Music Data Transmission Protocol on the soundcard.
 *
 * Copyright (C) 2002, Christian Gemeiner,
 *                     Manfred Weihs <weihs@ict.tuwien.ac.at>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <pthread.h>
#include "playamdtpmodule.h"

#define MAX_CHAR 500

static pthread_t player;
static int exittool = 0;

/*Prints the statistic of the soundcard buffer on the screen*/
static void statfct(void)
{
    if (running)
    {
	printf("\nTotal size of Buffer(bytes): %i\n", info.fragstotal*info.fragsize);
        printf("Current free bytes in Buffer: %i\n",info.bytes);
        printf("Current free space in Buffer: %i%%\n",bufsize);  
        fflush(stdout);
    }
    else printf("You can only see statistic if you listen!\n");

    
}

/*Changes the channelnumber to the user specified value*/
static void channelfct(void)
{
    char inputnr[MAX_CHAR];
    char *eptr;
    int returnche;    
    long value;
    int exitchannel=0;   
 
    do
    {
       returnche=0;
       (void) printf("\nChannelnumber (0 - 63): ");
       
       if (fgets(inputnr,MAX_CHAR,stdin) != NULL)    
       {
	   value=strtol(inputnr,&eptr,10);
           if ((eptr==inputnr)|| (*eptr != '\n'))
           { 
	       returnche=1;
               (void) printf("Channelnumber 0 - 63 !!");
               continue;
           }
           if ((value < 0)||(value > 63))
           {    
               returnche=1;
               (void) printf("Channelnumber 0 - 63 !! ");
           }
       }
       
       else 
       {  
         exitchannel=1;
         printf("\n");
       }
       
     }while(returnche); 
        
    if (!exitchannel)
    {
	if (!running) isochannel = (unsigned int) value;
        else
        { 
	    running=0;  
            pthread_join(player,NULL);
            isochannel = (unsigned int) value;
            running=1;
            pthread_create(&player, NULL, play, NULL);
        }
    }
   
}

/* Prints some help on the screen*/
static void helpfct(void)

{
    
    printf("\n************** HELP ***********************\n");
    printf("*                                         *\n");
    printf("*   start      command to start listen    *\n");
    printf("*   stop       command to stop listen     *\n");
    printf("*   exit       command to close tool      *\n");
    printf("*   help       command to see this text   *\n");
    printf("*   channel    command to set channelnr   *\n");
    printf("*              default=63,after you put   *\n");
    printf("*              in channel you are asked   *\n");
    printf("*              to put in the number 0-63  *\n");
    printf("*              with strg-d instead of nr  *\n");
    printf("*              you come back              *\n");
    printf("*   statistic  command to see values of   *\n");
    printf("*              the audiobuffer            *\n");
    printf("*                                         *\n");
    printf("************** HELP ***********************\n\n");
    fflush(stdout); 
}


/*Terminates the tool correctly*/
static void exitfct(void)
{
    if (running)
    {
	running=0;
        pthread_join(player,NULL);
    }
    exittool=1;  

}

/*Starts the playmodul as a own thread*/
static void startfct(void)
{
    
    if (!running)
    {
    
       running=1;
       pthread_create(&player, NULL, play, NULL);       
 
    }

    else fprintf(stderr,"You are already listen\n");

}

/*Terminates the playmodule*/
static void stopfct(void)
{
    
    if (running) 
    {   
       running=0; 
       pthread_join(player,NULL);
    }

    else fprintf(stderr,"You don't listen\n");   
}

/*Starts the function which is called by the command of the user*/
static void commandselect(char * command)
{ 
    int validcommand=0;
    int a;    
 
    if ((a=strcmp(command,"start"))==0)
    {
        startfct();
        validcommand=1;
    }
    if ((a=strcmp(command,"exit"))==0)
    {  
        exitfct();
        validcommand=1;
    }
    if ((a=strcmp(command,"help"))==0)  
    {
	helpfct();
        validcommand=1;
    }	
    if ((a=strcmp(command,"stop"))==0) 
    {
        stopfct();
        validcommand=1;
    }
    if ((a=strcmp(command,"channel"))==0)
    {  
        channelfct();
        validcommand=1;
    } 
    if ((a=strcmp(command,"statistic"))==0)
    {
        statfct();
        validcommand=1;
    } 
    
    if (!validcommand) fprintf(stderr,"Invalid Command\n");

}


/* Tells the user the correct tool call, if he used too many parameter */
void Usage(const char * command)
{
    (void) fprintf(stderr,"Usage: %s\n",command);
    exit(EXIT_FAILURE);
}


int main(int argc,char** argv)
{
   
    char input[MAX_CHAR];
    char *command;

    if (argc != 1) Usage(argv[0]);
    (void) printf("\nCommand: ");
    
    /* Reads the user command from the inputline and copies it to the "commandselect" function */
    /* if it is required, it terminates the tool correctly*/
    
    while ((!exittool)&&(fgets(input,MAX_CHAR,stdin) != NULL))
    {
	if ((command=strtok(input,"\n")) != NULL) commandselect(command);      
        if (!exittool) (void) printf("Command: ");   
    }
    if (running)
    {
	running=0;
        pthread_join(player,NULL);
    }

    return 0;


}
